@@ -61,7 +61,7 @@ class CandidateDeletion(Candidate):
     def __init__(self, source_contig, source_start, source_end, members, score, std_span, std_pos, support_fraction = ".", genotype = "./.", ref_reads = None, alt_reads = None):
         self.source_contig = source_contig
         #0-based start of the deletion (first deleted base)
-        self.source_start = source_start
+        self.source_start = max(0, source_start)
         #0-based end of the deletion (one past the last deleted base)
         self.source_end = source_end
 
@@ -114,7 +114,7 @@ class CandidateDeletion(Candidate):
                 info_string += ";ZMWS={0}".format(len(zmw_list))
         return "{chrom}\t{pos}\t{id}\t{ref}\t{alt}\t{qual}\t{filter}\t{info}\t{format}\t{samples}".format(
                     chrom=contig,
-                    pos=start,
+                    pos=max(1, start),
                     id="PLACEHOLDERFORID",
                     ref=ref_allele,
                     alt=alt_allele,
@@ -129,7 +129,7 @@ class CandidateInversion(Candidate):
     def __init__(self, source_contig, source_start, source_end, members, score, std_span, std_pos, support_fraction = ".", genotype = "./.", ref_reads = None, alt_reads = None):
         self.source_contig = source_contig
         #0-based start of the inversion (first inverted base)
-        self.source_start = source_start
+        self.source_start = max(0, source_start)
         #0-based end of the inversion (one past the last inverted base)
         self.source_end = source_end
 
@@ -195,12 +195,13 @@ class CandidateInversion(Candidate):
 
 
 class CandidateNovelInsertion(Candidate):
-    def __init__(self, dest_contig, dest_start, dest_end, members, score, std_span, std_pos, support_fraction = ".", genotype = "./.", ref_reads = None, alt_reads = None):
+    def __init__(self, dest_contig, dest_start, dest_end, sequence, members, score, std_span, std_pos, support_fraction = ".", genotype = "./.", ref_reads = None, alt_reads = None):
         self.dest_contig = dest_contig
         #0-based start of the insertion (base after the insertion)
-        self.dest_start = dest_start
+        self.dest_start = max(0, dest_start)
         #0-based start of the insertion (base after the insertion) + length of the insertion
         self.dest_end = dest_end
+        self.sequence = sequence
 
         self.members = members
         self.score = score
@@ -227,14 +228,12 @@ class CandidateNovelInsertion(Candidate):
         filters = []
         if self.genotype == "0/0":
             filters.append("hom_ref")
-        if sequence_alleles:
+        if sequence_alleles and self.sequence != "":
             ref_allele = reference.fetch(contig, max(0, start-1), max(0, start-1) + 1).upper()
-            #Use insertion sequence of a random read
-            alt_allele = ref_allele + self.members[0].sequence.upper()
+            alt_allele = ref_allele + self.sequence
         else:
             ref_allele = "N"
             alt_allele = "<" + self.type + ">"
-        info_template="SVTYPE={0};END={1};SUPPORT={2};STD_SPAN={3};STD_POS={4}"
         info_template="SVTYPE={0};END={1};SVLEN={2};SUPPORT={3};STD_SPAN={4};STD_POS={5}"
         info_string = info_template.format(self.type, 
                                            start, 
@@ -261,7 +260,7 @@ class CandidateNovelInsertion(Candidate):
                 info_string += ";ZMWS={0}".format(len(zmw_list))
         return "{chrom}\t{pos}\t{id}\t{ref}\t{alt}\t{qual}\t{filter}\t{info}\t{format}\t{samples}".format(
                     chrom=contig,
-                    pos=start,
+                    pos=max(1, start),
                     id="PLACEHOLDERFORID",
                     ref=ref_allele,
                     alt=alt_allele,
@@ -276,10 +275,11 @@ class CandidateDuplicationTandem(Candidate):
     def __init__(self, source_contig, source_start, source_end, copies, fully_covered, members, score, std_span, std_pos, support_fraction = ".", genotype = "./.", ref_reads = None, alt_reads = None):
         self.source_contig = source_contig
         #0-based start of the region (first copied base)
-        self.source_start = source_start
+        self.source_start = max(0, source_start)
         #0-based end of the region (one past the last copied base)
         self.source_end = source_end
         
+        #number of copies inserted after end of the region (in tandem)
         self.copies = copies
 
         self.members = members
@@ -320,10 +320,9 @@ class CandidateDuplicationTandem(Candidate):
         return (source_entry, dest_entry)
 
 
-    def get_vcf_entry_as_ins(self, read_names = False, zmws = False):
-        contig = self.source_contig
-        start = self.source_end
-        end = self.source_end + self.copies * (self.source_end - self.source_start)
+    def get_vcf_entry_as_ins(self, sequence_alleles = False, reference = None, read_names = False, zmws = False):
+        source_contig, source_start, source_end = self.get_source()
+        dest_contig, dest_start, dest_end = self.get_destination()
         svtype = "INS"
         if self.ref_reads != None and self.alt_reads != None:
             dp_string = str(self.ref_reads + self.alt_reads)
@@ -332,14 +331,20 @@ class CandidateDuplicationTandem(Candidate):
         filters = []
         if self.genotype == "0/0":
             filters.append("hom_ref")
-        if not(self.fully_covered):
+        if not self.fully_covered:
             filters.append("not_fully_covered")
+        if sequence_alleles:
+            ref_allele = reference.fetch(source_contig, source_start, source_end).upper()
+            alt_allele = ref_allele * (self.copies + 1)
+        else:
+            ref_allele = "N"
+            alt_allele = "<" + self.type + ">"
         info_template="SVTYPE={0};END={1};SVLEN={2};SUPPORT={3};STD_SPAN={4};STD_POS={5}"
-        info_string = info_template.format(svtype, 
-                                           start, 
-                                           end - start, 
-                                           len(set([sig.read for sig in self.members])), 
-                                           self.get_std_span(), 
+        info_string = info_template.format(svtype,
+                                           source_end,
+                                           dest_end - dest_start,
+                                           len(set([sig.read for sig in self.members])),
+                                           self.get_std_span(),
                                            self.get_std_pos())
         read_ids = [member.read for member in self.members]
         if read_names:
@@ -356,11 +361,11 @@ class CandidateDuplicationTandem(Candidate):
             if valid_pacbio_names:
                 info_string += ";ZMWS={0}".format(len(zmw_list))
         return "{chrom}\t{pos}\t{id}\t{ref}\t{alt}\t{qual}\t{filter}\t{info}\t{format}\t{samples}".format(
-                    chrom=contig,
-                    pos=start,
+                    chrom=source_contig,
+                    pos=source_start + 1,
                     id="PLACEHOLDERFORID",
-                    ref="N",
-                    alt="<" + svtype + ">",
+                    ref=ref_allele,
+                    alt=alt_allele,
                     qual=int(self.score),
                     filter="PASS" if len(filters) == 0 else ";".join(filters),
                     info=info_string,
@@ -406,7 +411,7 @@ class CandidateDuplicationTandem(Candidate):
                 info_string += ";ZMWS={0}".format(len(zmw_list))
         return "{chrom}\t{pos}\t{id}\t{ref}\t{alt}\t{qual}\t{filter}\t{info}\t{format}\t{samples}".format(
                     chrom=contig,
-                    pos=start,
+                    pos=start+1,
                     id="PLACEHOLDERFORID",
                     ref="N",
                     alt="<" + svtype + ">",
@@ -421,13 +426,13 @@ class CandidateDuplicationInterspersed(Candidate):
     def __init__(self, source_contig, source_start, source_end, dest_contig, dest_start, dest_end, members, score, std_span, std_pos, cutpaste=False, support_fraction = ".", genotype = "./.", ref_reads = None, alt_reads = None):
         self.source_contig = source_contig
         #0-based start of the region (first copied base)
-        self.source_start = source_start
+        self.source_start = max(0, source_start)
         #0-based end of the region (one past the last copied base)
         self.source_end = source_end
 
         self.dest_contig = dest_contig
         #0-based start of the insertion (base after the insertion)
-        self.dest_start = dest_start
+        self.dest_start = max(0, dest_start)
         #0-based end of the insertion (base after the insertion) + length of the insertion
         self.dest_end = dest_end
 
@@ -468,8 +473,9 @@ class CandidateDuplicationInterspersed(Candidate):
         return (source_entry, dest_entry)
 
 
-    def get_vcf_entry_as_ins(self, read_names = False, zmws = False):
-        contig, start, end = self.get_destination()
+    def get_vcf_entry_as_ins(self, sequence_alleles = False, reference = None, read_names = False, zmws = False):
+        source_contig, source_start, source_end = self.get_source()
+        dest_contig, dest_start, dest_end = self.get_destination()
         svtype = "INS"
         if self.ref_reads != None and self.alt_reads != None:
             dp_string = str(self.ref_reads + self.alt_reads)
@@ -478,13 +484,19 @@ class CandidateDuplicationInterspersed(Candidate):
         filters = []
         if self.genotype == "0/0":
             filters.append("hom_ref")
+        if sequence_alleles:
+            ref_allele = reference.fetch(dest_contig, max(0, dest_start-1), max(0, dest_start-1) + 1).upper()
+            alt_allele = ref_allele + reference.fetch(source_contig, source_start, source_end).upper()
+        else:
+            ref_allele = "N"
+            alt_allele = "<" + self.type + ">"
         info_template="SVTYPE={0};{1}END={2};SVLEN={3};SUPPORT={4};STD_SPAN={5};STD_POS={6}"
-        info_string = info_template.format(svtype, 
-                                           "CUTPASTE;" if self.cutpaste else "", 
-                                           start, 
-                                           end - start, 
-                                           len(set([sig.read for sig in self.members])), 
-                                           self.get_std_span(), 
+        info_string = info_template.format(svtype,
+                                           "CUTPASTE;" if self.cutpaste else "",
+                                           dest_start,
+                                           dest_end - dest_start,
+                                           len(set([sig.read for sig in self.members])),
+                                           self.get_std_span(),
                                            self.get_std_pos())
         read_ids = [member.read for member in self.members]
         if read_names:
@@ -501,11 +513,11 @@ class CandidateDuplicationInterspersed(Candidate):
             if valid_pacbio_names:
                 info_string += ";ZMWS={0}".format(len(zmw_list))
         return "{chrom}\t{pos}\t{id}\t{ref}\t{alt}\t{qual}\t{filter}\t{info}\t{format}\t{samples}".format(
-                    chrom=contig,
-                    pos=start,
+                    chrom=dest_contig,
+                    pos=max(1, dest_start),
                     id="PLACEHOLDERFORID",
-                    ref="N",
-                    alt="<" + svtype + ">",
+                    ref=ref_allele,
+                    alt=alt_allele,
                     qual=int(self.score),
                     filter="PASS" if len(filters) == 0 else ";".join(filters),
                     info=info_string,
@@ -547,7 +559,7 @@ class CandidateDuplicationInterspersed(Candidate):
                 info_string += ";ZMWS={0}".format(len(zmw_list))
         return "{chrom}\t{pos}\t{id}\t{ref}\t{alt}\t{qual}\t{filter}\t{info}\t{format}\t{samples}".format(
                     chrom=contig,
-                    pos=start,
+                    pos=start+1,
                     id="PLACEHOLDERFORID",
                     ref="N",
                     alt="<" + svtype + ">",
@@ -562,12 +574,12 @@ class CandidateBreakend(Candidate):
     def __init__(self, source_contig, source_start, source_direction, dest_contig, dest_start, dest_direction, members, score, std_pos1, std_pos2, support_fraction = ".", genotype = "./.", ref_reads = None, alt_reads = None):
         self.source_contig = source_contig
         #0-based source of the translocation (first base before the translocation)
-        self.source_start = source_start
+        self.source_start = max(0, source_start)
         self.source_direction = source_direction
 
         self.dest_contig = dest_contig
         #0-based destination of the translocation (first base after the translocation)
-        self.dest_start = dest_start
+        self.dest_start = max(0, dest_start)
         self.dest_direction = dest_direction
 
         self.members = members
@@ -629,13 +641,13 @@ class CandidateBreakend(Candidate):
         source_contig, source_start = self.get_source()
         dest_contig, dest_start = self.get_destination()
         if (self.source_direction == 'fwd') and (self.dest_direction == 'fwd'):
-            alt_string = "N[{contig}:{start}[".format(contig = dest_contig, start = dest_start)
+            alt_string = "N[{contig}:{start}[".format(contig = dest_contig, start = dest_start+1)
         elif (self.source_direction == 'fwd') and (self.dest_direction == 'rev'):
-            alt_string = "N]{contig}:{start}]".format(contig = dest_contig, start = dest_start)
+            alt_string = "N]{contig}:{start}]".format(contig = dest_contig, start = dest_start+1)
         elif (self.source_direction == 'rev') and (self.dest_direction == 'rev'):
-            alt_string = "]{contig}:{start}]N".format(contig = dest_contig, start = dest_start)
+            alt_string = "]{contig}:{start}]N".format(contig = dest_contig, start = dest_start+1)
         elif (self.source_direction == 'rev') and (self.dest_direction == 'fwd'):
-            alt_string = "[{contig}:{start}[N".format(contig = dest_contig, start = dest_start)
+            alt_string = "[{contig}:{start}[N".format(contig = dest_contig, start = dest_start+1)
         if self.ref_reads != None and self.alt_reads != None:
             dp_string = str(self.ref_reads + self.alt_reads)
         else:
@@ -664,7 +676,57 @@ class CandidateBreakend(Candidate):
                 info_string += ";ZMWS={0}".format(len(zmw_list))
         return "{chrom}\t{pos}\t{id}\t{ref}\t{alt}\t{qual}\t{filter}\t{info}\t{format}\t{samples}".format(
                     chrom=source_contig,
-                    pos=source_start,
+                    pos=source_start+1,
+                    id="PLACEHOLDERFORID",
+                    ref="N",
+                    alt=alt_string,
+                    qual=int(self.score),
+                    filter="PASS" if len(filters) == 0 else ";".join(filters),
+                    info=info_string,
+                    format="GT:DP:AD",
+                    samples="{gt}:{dp}:{ref},{alt}".format(gt=self.genotype, dp=dp_string, ref=self.ref_reads if self.ref_reads != None else ".", alt=self.alt_reads if self.alt_reads != None else "."))
+
+
+    def get_vcf_entry_reverse(self, read_names = False, zmws = False):
+        source_contig, source_start = self.get_destination()
+        dest_contig, dest_start = self.get_source()
+        if (self.source_direction == 'rev') and (self.dest_direction == 'rev'):
+            alt_string = "N[{contig}:{start}[".format(contig = dest_contig, start = dest_start+1)
+        elif (self.source_direction == 'fwd') and (self.dest_direction == 'rev'):
+            alt_string = "N]{contig}:{start}]".format(contig = dest_contig, start = dest_start+1)
+        elif (self.source_direction == 'fwd') and (self.dest_direction == 'fwd'):
+            alt_string = "]{contig}:{start}]N".format(contig = dest_contig, start = dest_start+1)
+        elif (self.source_direction == 'rev') and (self.dest_direction == 'fwd'):
+            alt_string = "[{contig}:{start}[N".format(contig = dest_contig, start = dest_start+1)
+        if self.ref_reads != None and self.alt_reads != None:
+            dp_string = str(self.ref_reads + self.alt_reads)
+        else:
+            dp_string = "."
+        filters = []
+        if self.genotype == "0/0":
+            filters.append("hom_ref")
+        info_template="SVTYPE={0};SUPPORT={1};STD_POS1={2};STD_POS2={3}"
+        info_string = info_template.format(self.type, 
+                                           len(set([sig.read for sig in self.members])), 
+                                           self.get_std_pos2(), 
+                                           self.get_std_pos1())
+        read_ids = [member.read for member in self.members]
+        if read_names:
+            info_string += ";READS={0}".format(",".join(read_ids))
+        if zmws:
+            valid_pacbio_names = True
+            zmw_list = set()
+            for read_id in read_ids:
+                fields = read_id.split("/")
+                if len(fields) != 3:
+                    valid_pacbio_names = False
+                    break
+                zmw_list.add("/".join(fields[0:2]))
+            if valid_pacbio_names:
+                info_string += ";ZMWS={0}".format(len(zmw_list))
+        return "{chrom}\t{pos}\t{id}\t{ref}\t{alt}\t{qual}\t{filter}\t{info}\t{format}\t{samples}".format(
+                    chrom=source_contig,
+                    pos=source_start+1,
                     id="PLACEHOLDERFORID",
                     ref="N",
                     alt=alt_string,
