@@ -7,18 +7,12 @@ from math import pow, sqrt
 import time
 from statistics import mean, stdev
 from pysam import FastaFile
+from spoa import poa
+from cpuinfo import get_cpu_info
 
 from svim.SVIM_clustering import form_partitions, partition_and_cluster_candidates, calculate_score
 from svim.SVCandidate import CandidateInversion, CandidateDuplicationTandem, CandidateDeletion, CandidateNovelInsertion, CandidateBreakend
-from svim.SVIM_merging import flag_cutpaste_candidates, merge_translocations_at_insertions, cluster_positions_simple
-
-
-def cluster_sv_candidates(int_duplication_candidates, options):
-    """Cluster SVCandidates to remove redundancy"""
-
-    final_int_duplication_candidates = partition_and_cluster_candidates(int_duplication_candidates, options, "interspersed duplication candidates")
-
-    return final_int_duplication_candidates
+from svim.SVIM_merging import flag_cutpaste_candidates, merge_translocations_at_insertions
 
 
 def write_candidates(working_dir, candidates):
@@ -133,7 +127,7 @@ def write_final_vcf(int_duplication_candidates,
     print("#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\tFORMAT\t" + options.sample, file=vcf_output)
 
     # Open reference genome sequence file
-    sequence_alleles = options.sequence_alleles
+    sequence_alleles = not options.symbolic_alleles
     if sequence_alleles:
         try:
             reference = FastaFile(options.genome)
@@ -160,7 +154,7 @@ def write_final_vcf(int_duplication_candidates,
     if options.tandem_duplications_as_insertions:
         if "INS" in types_to_output:
             for candidate in tandem_duplication_candidates:
-                vcf_entries.append((candidate.get_destination(), candidate.get_vcf_entry_as_ins(options.read_names, options.zmws), "INS"))
+                vcf_entries.append((candidate.get_destination(), candidate.get_vcf_entry_as_ins(sequence_alleles, reference, options.read_names, options.zmws), "INS"))
     else:
         if "DUP:TANDEM" in types_to_output:
             for candidate in tandem_duplication_candidates:
@@ -168,7 +162,7 @@ def write_final_vcf(int_duplication_candidates,
     if options.interspersed_duplications_as_insertions:
         if "INS" in types_to_output:
             for candidate in int_duplication_candidates:
-                vcf_entries.append((candidate.get_destination(), candidate.get_vcf_entry_as_ins(options.read_names, options.zmws), "INS"))
+                vcf_entries.append((candidate.get_destination(), candidate.get_vcf_entry_as_ins(sequence_alleles, reference, options.read_names, options.zmws), "INS"))
     else:
         if "DUP:INT" in types_to_output:
             for candidate in int_duplication_candidates:
@@ -176,6 +170,7 @@ def write_final_vcf(int_duplication_candidates,
     if "BND" in types_to_output:
         for candidate in breakend_candidates:
             vcf_entries.append(((candidate.get_source()[0], candidate.get_source()[1], candidate.get_source()[1] + 1), candidate.get_vcf_entry(options.read_names, options.zmws), "BND"))
+            vcf_entries.append(((candidate.get_destination()[0], candidate.get_destination()[1], candidate.get_destination()[1] + 1), candidate.get_vcf_entry_reverse(options.read_names, options.zmws), "BND"))
 
     if sequence_alleles:
         reference.close()
@@ -190,9 +185,152 @@ def write_final_vcf(int_duplication_candidates,
 
     vcf_output.close()
 
+def generate_insertion_consensus(ins_cluster, reference, window_padding = 100, maximum_haplotype_length = 10000, allowed_size_deviation = 2.0):
+    #compute window containing all members of cluster
+    member_pos = [member.start for member in ins_cluster.members]
+    window_start = min(member_pos) - window_padding
+    window_end = max(member_pos) + window_padding
+
+    #construct haplotype sequences from all reads
+    haplotypes = []
+    for member in ins_cluster.members:
+        haplotype_sequence = reference.fetch(ins_cluster.contig, max(0, window_start), max(0, member.start)).upper()
+        haplotype_sequence += member.sequence.upper()
+        haplotype_sequence += reference.fetch(ins_cluster.contig, max(0, member.start), max(0, window_end)).upper()
+        haplotypes.append(haplotype_sequence)
+    largest_haplotype_length = max([len(h) for h in haplotypes])
+    if largest_haplotype_length > maximum_haplotype_length:
+        logging.info("Skipping consensus computation for insertion with haplotypes exceeding maximum length ({0} > {1})".format(largest_haplotype_length, maximum_haplotype_length))
+        return (1, ())
+
+    #compute consensus of all reads using SPOA
+    try:
+        consensus_reads, msa_reads = poa(haplotypes, algorithm=1, m=2, n=-4, g=-4, e=-2, q=-24, c=-1)
+    except MemoryError:
+        logging.warning("Error: spoa ran out of memory while computing a consensus sequence for a cluster of insertion signatures (size = {0}, maximum haplotype length = {1}).".format(ins_cluster.size, largest_haplotype_length))
+        return (2, ())
+    except ValueError as ve:
+        logging.warning("Error: spoa failed while computing a consensus sequence for a cluster of insertion signatures (size = {0}, maximum haplotype length = {1}): {2}".format(ins_cluster.size, largest_haplotype_length, ve))
+        return (2, ())
+
+    #re-align consensus sequence to reference sequence in the window
+    ref_sequence = reference.fetch(ins_cluster.contig, max(0, window_start), max(0, window_end)).upper()
+    try:
+        consensus_reads_ref, msa_reads_ref = poa([consensus_reads, ref_sequence], algorithm=1, m=2, n=-4, g=-4, e=-2, q=-24, c=-1)
+    except MemoryError:
+        logging.warning("Error: spoa ran out of memory while aligning the consensus sequence for a cluster of insertion signatures (size = {0}, maximum haplotype length = {1}).".format(ins_cluster.size, largest_haplotype_length))
+        return (2, ())
+    except ValueError as ve:
+        logging.warning("Error: spoa failed while aligning the consensus sequence for a cluster of insertion signatures (size = {0}, maximum haplotype length = {1}): {2}".format(ins_cluster.size, largest_haplotype_length, ve))
+        return (2, ())
+
+    #locate insertion relative to reference and check whether size is close to expected size
+    expected_size = ins_cluster.end - ins_cluster.start
+    matches = []
+    for match in re.finditer(r'-+', msa_reads_ref[1]):
+        match_size = match.end() - match.start()
+        size_ratio = max(match_size, expected_size) / min(match_size, expected_size)
+        matches.append((match.start(), match_size, size_ratio))
+    good_matches = [m for m in matches if m[2] < allowed_size_deviation]
+    if len(good_matches) == 0:
+        logging.info("Consensus failure (no suitable insertion found in realignment step). \
+                      Expected size: {0}; Match sizes: {1}; Ref: {2}; Consensus: {3}".format(expected_size,
+                                                                                             "/".join([str(m[1]) for m in matches]),
+                                                                                             msa_reads_ref[1],
+                                                                                             msa_reads_ref[0]))
+        return (3, ())
+    elif len(good_matches) == 1:
+        realigned_insertion_start = max(0, window_start) + good_matches[0][0]
+        realigned_insertion_size = good_matches[0][1]
+        insertion_consensus = msa_reads_ref[0][good_matches[0][0]:good_matches[0][0]+good_matches[0][1]]
+        logging.debug("Consensus success. Expected size: {0}; Consensus size: {1}; Consensus insertion: {2}".format(expected_size, realigned_insertion_size, insertion_consensus))
+        return (0, (realigned_insertion_start, realigned_insertion_size, insertion_consensus))
+    else:
+        logging.info("Consensus failure (multiple suitable insertions found in realignment step). \
+                      Expected size: {0}; Match sizes: {1}; Ref: {2}; Consensus: {3}".format(expected_size,
+                                                                                             "/".join([str(m[1]) for m in matches]),
+                                                                                             msa_reads_ref[1],
+                                                                                             msa_reads_ref[0]))
+        return (4, ())
+
+
+def prepare_insertion_candidates(insertion_signature_clusters, options):
+    novel_insertion_candidates = []
+    if options.skip_consensus or ("sse4_1" not in get_cpu_info()["flags"]):
+        if options.skip_consensus:
+            logging.info("Skipping computation of insertion consensus sequences because of --skip_consensus flag.")
+        else:
+            logging.warning("Skipping computation of insertion consensus sequences "
+                            "because CPU does not support SSE 4.1 instruction set.")
+        for ins_cluster in insertion_signature_clusters:
+            if ins_cluster.score > 0:
+                novel_insertion_candidates.append(CandidateNovelInsertion(ins_cluster.contig,
+                                                                              ins_cluster.start,
+                                                                              ins_cluster.end,
+                                                                              "",
+                                                                              ins_cluster.members,
+                                                                              ins_cluster.score,
+                                                                              ins_cluster.std_span,
+                                                                              ins_cluster.std_pos))
+        return novel_insertion_candidates
+    logging.info("Generating and realigning consensus sequence for insertions..")
+    with FastaFile(options.genome) as reference:
+        # 0-successful, 1-skipped, 2-failed, 3-no consensus, 4-multiple consensuses
+        status_counter = [0, 0, 0, 0, 0]
+        for ins_cluster in insertion_signature_clusters:
+            if ins_cluster.score > 0:
+                if len(ins_cluster.members) < 3:
+                    novel_insertion_candidates.append(CandidateNovelInsertion(ins_cluster.contig,
+                                                                              ins_cluster.start,
+                                                                              ins_cluster.end,
+                                                                              ins_cluster.members[0].sequence,
+                                                                              ins_cluster.members,
+                                                                              ins_cluster.score,
+                                                                              ins_cluster.std_span,
+                                                                              ins_cluster.std_pos))
+                    continue
+                status, consensus_result = generate_insertion_consensus(ins_cluster,
+                                                                        reference,
+                                                                        maximum_haplotype_length=options.max_consensus_length)
+                try:
+                    status_counter[status] += 1
+                except KeyError:
+                    logging.error("Unknown status {0} returned by consensus computation.".format(status))
+                    continue
+                # Successful
+                if status == 0:
+                    realigned_insertion_start, realigned_insertion_size, insertion_consensus = consensus_result
+                    novel_insertion_candidates.append(CandidateNovelInsertion(ins_cluster.contig,
+                                                                              realigned_insertion_start,
+                                                                              realigned_insertion_start + realigned_insertion_size,
+                                                                              insertion_consensus,
+                                                                              ins_cluster.members,
+                                                                              ins_cluster.score,
+                                                                              ins_cluster.std_span,
+                                                                              ins_cluster.std_pos))
+                #Unsuccessful
+                else:
+                    novel_insertion_candidates.append(CandidateNovelInsertion(ins_cluster.contig,
+                                                                              ins_cluster.start,
+                                                                              ins_cluster.end,
+                                                                              "",
+                                                                              ins_cluster.members,
+                                                                              ins_cluster.score,
+                                                                              ins_cluster.std_span,
+                                                                              ins_cluster.std_pos))
+    message = ("Generated and realigned consensus sequences for {0} insertions "
+               "({1} skipped, {2} failed with an error, {3} failed with no "
+               "consensus, {4} failed with multiple consensuses).")
+    logging.info(message.format(status_counter[0],
+                                status_counter[1],
+                                status_counter[2],
+                                status_counter[3],
+                                status_counter[4]))
+    return novel_insertion_candidates
+
 
 def combine_clusters(signature_clusters, options):
-    deletion_signature_clusters, insertion_signature_clusters, inversion_signature_clusters, tandem_duplication_signature_clusters, insertion_from_signature_clusters, completed_translocations = signature_clusters
+    deletion_signature_clusters, insertion_signature_clusters, inversion_signature_clusters, tandem_duplication_signature_clusters, insertion_from_signature_clusters, translocation_signature_clusters = signature_clusters
 
     ###############################
     # Create inversion candidates #
@@ -217,77 +355,39 @@ def combine_clusters(signature_clusters, options):
     #####################################
 
     # Cluster translocations by contig and pos1
-    logging.info("Cluster translocation breakpoints..")
-    translocations_fwdfwd = [tra for tra in completed_translocations if tra.direction1 == "fwd" and tra.direction2 == "fwd"]
-    translocations_revrev = [tra for tra in completed_translocations if tra.direction1 == "rev" and tra.direction2 == "rev"]
-    translocations_fwdrev = [tra for tra in completed_translocations if tra.direction1 == "fwd" and tra.direction2 == "rev"]
-    translocations_revfwd = [tra for tra in completed_translocations if tra.direction1 == "rev" and tra.direction2 == "fwd"]
-    translocation_partitions_fwdfwd = form_partitions(translocations_fwdfwd, options.trans_partition_max_distance)
-    translocation_partitions_revrev = form_partitions(translocations_revrev, options.trans_partition_max_distance)
-    translocation_partitions_fwdrev = form_partitions(translocations_fwdrev, options.trans_partition_max_distance)
-    translocation_partitions_revfwd = form_partitions(translocations_revfwd, options.trans_partition_max_distance)
+    # logging.info("Cluster translocation breakpoints..")
+    # translocations_fwdfwd = [tra for tra in translocation_signature_clusters if tra.direction1 == "fwd" and tra.direction2 == "fwd"]
+    # translocations_revrev = [tra for tra in translocation_signature_clusters if tra.direction1 == "rev" and tra.direction2 == "rev"]
+    # translocations_fwdrev = [tra for tra in translocation_signature_clusters if tra.direction1 == "fwd" and tra.direction2 == "rev"]
+    # translocations_revfwd = [tra for tra in translocation_signature_clusters if tra.direction1 == "rev" and tra.direction2 == "fwd"]
+    # translocation_partitions_fwdfwd = form_partitions(translocations_fwdfwd, options.trans_partition_max_distance)
+    # translocation_partitions_revrev = form_partitions(translocations_revrev, options.trans_partition_max_distance)
+    # translocation_partitions_fwdrev = form_partitions(translocations_fwdrev, options.trans_partition_max_distance)
+    # translocation_partitions_revfwd = form_partitions(translocations_revfwd, options.trans_partition_max_distance)
 
     ##############################
     # Create breakend candidates #
     ##############################
 
     breakend_candidates = []
-    for (dir1, dir2), partitions in {('fwd', 'fwd'): translocation_partitions_fwdfwd,
-                      ('fwd', 'rev'): translocation_partitions_fwdrev,
-                      ('rev', 'rev'): translocation_partitions_revrev,
-                      ('rev', 'fwd'): translocation_partitions_revfwd}.items():
-        for partition in partitions:
-            destination_partitions = cluster_positions_simple([(signature.contig2, signature.pos2, signature) for signature in partition], \
-                                                                    options.trans_destination_partition_max_distance)
-            for destination_partition in destination_partitions:
-                contig1 = partition[0].contig1
-                pos1_mean = int(round(mean([signature.pos1 for contig2, pos2, signature in destination_partition])))
-                contig2 = destination_partition[0][0]
-                pos2_mean = int(round(mean([pos2 for contig2, pos2, signature in destination_partition])))
-                if len(destination_partition) > 1:
-                    pos1_std = int(round(stdev([signature.pos1 for contig2, pos2, signature in destination_partition])))
-                    pos2_std = int(round(stdev([pos2 for contig2, pos2, signature in destination_partition])))
-                else:
-                    pos1_std = None
-                    pos2_std = None
-                members = [signature for contig2, pos2, signature in destination_partition]
-                #Use pos1_std and pos2_std because breakends do not have a span (for span_std)
-                score = calculate_score(members, pos1_std, pos2_std, options.trans_partition_max_distance, "BND")
-                breakend_candidates.append(CandidateBreakend(contig1, 
-                                                            pos1_mean, 
-                                                            dir1, 
-                                                            contig2, 
-                                                            pos2_mean, 
-                                                            dir2, 
-                                                            members, 
-                                                            score, 
-                                                            pos1_std, 
-                                                            pos2_std))
+    for tra_cluster in translocation_signature_clusters:
+        breakend_candidates.append(CandidateBreakend(tra_cluster.source_contig, 
+                                                     tra_cluster.source_start, 
+                                                     tra_cluster.direction1, 
+                                                     tra_cluster.dest_contig, 
+                                                     tra_cluster.dest_start, 
+                                                     tra_cluster.direction2, 
+                                                     tra_cluster.members, 
+                                                     tra_cluster.score, 
+                                                     tra_cluster.std_span, 
+                                                     tra_cluster.std_pos))
 
     ###################################################
     # Merge translocation breakpoints with insertions #
     ###################################################
 
-    translocation_partitions_fwdfwd_dict = defaultdict(list)
-    translocation_partitions_revrev_dict = defaultdict(list)
-    for partition in translocation_partitions_fwdfwd:
-        translocation_partitions_fwdfwd_dict[partition[0].contig1].append(partition)
-    for partition in translocation_partitions_revrev:
-        translocation_partitions_revrev_dict[partition[0].contig1].append(partition)
-
-    translocation_partition_means_fwdfwd_dict = {}
-    translocation_partition_stds_fwdfwd_dict = {}
-    for contig in translocation_partitions_fwdfwd_dict.keys():
-        translocation_partition_means_fwdfwd_dict[contig] = [int(round(sum([ev.pos1 for ev in partition]) / len(partition))) for partition in translocation_partitions_fwdfwd_dict[contig]]
-        translocation_partition_stds_fwdfwd_dict[contig] = [int(round(sqrt(sum([pow(abs(ev.pos1 - translocation_partition_means_fwdfwd_dict[contig][index]), 2) for ev in partition]) / len(partition)))) for index, partition in enumerate(translocation_partitions_fwdfwd_dict[contig])]
-    translocation_partition_means_revrev_dict = {}
-    translocation_partition_stds_revrev_dict = {}
-    for contig in translocation_partitions_revrev_dict.keys():
-        translocation_partition_means_revrev_dict[contig] = [int(round(sum([ev.pos1 for ev in partition]) / len(partition))) for partition in translocation_partitions_revrev_dict[contig]]
-        translocation_partition_stds_revrev_dict[contig] = [int(round(sqrt(sum([pow(abs(ev.pos1 - translocation_partition_means_revrev_dict[contig][index]), 2) for ev in partition]) / len(partition)))) for index, partition in enumerate(translocation_partitions_revrev_dict[contig])]
-
     logging.info("Combine inserted regions with translocation breakpoints..")
-    new_insertion_from_clusters, inserted_regions_to_remove_1 = merge_translocations_at_insertions(translocation_partitions_fwdfwd_dict, translocation_partition_means_fwdfwd_dict, translocation_partition_stds_fwdfwd_dict, translocation_partitions_revrev_dict, translocation_partition_means_revrev_dict, translocation_partition_stds_revrev_dict, insertion_signature_clusters, options)
+    new_insertion_from_clusters, inserted_regions_to_remove_1 = merge_translocations_at_insertions(translocation_signature_clusters, insertion_signature_clusters, options)
     insertion_from_signature_clusters.extend(new_insertion_from_clusters)
 
     ############################################################################
@@ -367,15 +467,12 @@ def combine_clusters(signature_clusters, options):
     #####################################
     # Create novel insertion candidates #
     #####################################
-    novel_insertion_candidates = []
-    for ins_cluster in insertion_signature_clusters:
-        if ins_cluster.score > 0:
-            novel_insertion_candidates.append(CandidateNovelInsertion(ins_cluster.contig, ins_cluster.start, ins_cluster.end, ins_cluster.members, ins_cluster.score, ins_cluster.std_span, ins_cluster.std_pos))
+    novel_insertion_candidates = prepare_insertion_candidates(insertion_signature_clusters, options)
 
     ######################
     # Cluster candidates #
     ######################
     logging.info("Cluster interspersed duplication candidates one more time..")
-    final_int_duplication_candidates = cluster_sv_candidates(int_duplication_candidates, options)
+    final_int_duplication_candidates = partition_and_cluster_candidates(int_duplication_candidates, options, "interspersed duplication candidates")
 
-    return (deletion_candidates, inversion_candidates, final_int_duplication_candidates, tan_dup_candidates, novel_insertion_candidates, breakend_candidates)
\ No newline at end of file
+    return (deletion_candidates, inversion_candidates, final_int_duplication_candidates, tan_dup_candidates, novel_insertion_candidates, breakend_candidates)
