@@ -7,8 +7,8 @@ import argparse
 def parse_arguments(program_version, arguments = sys.argv[1:]):
     parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                      description="""SVIM (pronounced SWIM) is a structural variant caller for long reads. 
-It discriminates five different variant classes: deletions, tandem and interspersed duplications, 
-inversions and insertions. SVIM is unique in its capability of extracting both the genomic origin and 
+It discriminates six different variant classes: deletions, tandem and interspersed duplications, 
+inversions, insertions and translocations. SVIM is unique in its capability of extracting both the genomic origin and 
 destination of duplications.
 
 SVIM consists of four major steps:
@@ -120,7 +120,7 @@ Alternatively, it can detect SVs from existing reads alignments in SAM/BAM forma
                                               determines the maximum distance between two subsequent signatures in the same partition. \
                                               If the distance between two subsequent signatures \
                                               is larger than this parameter, they are distributed into separate partitions.')
-    group_fasta_cluster.add_argument('--distance_normalizer',
+    group_fasta_cluster.add_argument('--position_distance_normalizer',
                                         type=int,
                                         default=900,
                                         help='Distance normalizer used for span-position distance (default: %(default)s). \
@@ -135,9 +135,24 @@ Alternatively, it can detect SVs from existing reads alignments in SAM/BAM forma
                                               A smaller distance normalizer leads to a higher position distance and as a \
                                               consequence increases the importance of the position distance in the \
                                               span-position distance relative to the span distance.')
+    group_fasta_cluster.add_argument('--edit_distance_normalizer',
+                                      type=float,
+                                      default=1.0,
+                                      help='Distance normalizer used specifically for insertions (default: %(default)s). \
+                                            SVIM clusters insertion signatures using an hierarchical clustering approach and a \
+                                            special distance metric for insertions. This distance is the sum of two \
+                                            components, position distance and edit distance between the insertion sequences. \
+                                            The edit distance is normalized (i.e. divided) by the product of the span of the \
+                                            longer insertion and this normalizer. The position distance is the difference in \
+                                            position between signatures normalized by the position distance normalizer \
+                                            (another parameter). A smaller edit distance normalizer leads to a larger \
+                                            edit distance and as a consequence increases the importance of the edit distance \
+                                            in the clustering process so that only insertions with very similar sequences \
+                                            are clustered together. A larger edit distance normalizer diminishes the importance \
+                                            of the insertion sequences in the clustering process.')
     group_fasta_cluster.add_argument('--cluster_max_distance',
                                         type=float,
-                                        default=0.3,
+                                        default=0.5,
                                         help='Maximum span-position distance between SVs in a cluster (default: %(default)s). \
                                               This is the most important parameter because it determines the strictness \
                                               of clustering. Choosing a large value leads to fewer but larger clusters with larger \
@@ -150,18 +165,25 @@ Alternatively, it can detect SVs from existing reads alignments in SAM/BAM forma
                                         type=float,
                                         default=1.0,
                                         help='Maximum span-position distance between the origin of an insertion and a deletion to be flagged as a potential cut&paste insertion (default: %(default)s)')
-    group_fasta_combine.add_argument('--trans_destination_partition_max_distance',
-                                        type=int,
-                                        default=1000,
-                                        help='Maximum distance in bp between translocation breakpoint destinations in a partition (default: %(default)s)')
-    group_fasta_combine.add_argument('--trans_partition_max_distance',
-                                        type=int,
-                                        default=200,
-                                        help='Maximum distance in bp between translocation breakpoints in a partition (default: %(default)s)')
     group_fasta_combine.add_argument('--trans_sv_max_distance',
                                         type=int,
                                         default=500,
                                         help='Maximum distance in bp between a translocation breakpoint and an SV signature to be combined (default: %(default)s)')
+    group_fasta_combine.add_argument('--skip_consensus',
+                                        action='store_true',
+                                        help='Disable consensus computation for insertions (default: %(default)s). \
+                                              This reduces the time and memory consumption of SVIM and might be \
+                                              useful if consensus sequences are not needed. With this option, \
+                                              insertion calls are represented by symbolic alleles (<INS>) instead \
+                                              of sequence alles in the output VCF. \
+                                              Consensus computation requires a modern CPU with the SSE 4.1 \
+                                              instruction set. For older CPUs missing this instruction set, \
+                                              consensus computation is automatically disabled.')
+    group_fasta_combine.add_argument('--max_consensus_length',
+                                        type=int,
+                                        default=10000,
+                                        help='Maximum size of insertion sequences for consensus computation. (default: %(default)s)\
+                                              For insertions longer than this threshold, no consensus is computed to save memory.')
     group_fasta_genotype = parser_fasta.add_argument_group('GENOTYPE')
     group_fasta_genotype.add_argument('--skip_genotyping',
                                         action='store_true',
@@ -208,16 +230,14 @@ Alternatively, it can detect SVs from existing reads alignments in SAM/BAM forma
                                               Give a comma-separated list of SV types. The possible SV types are: DEL (deletions), \
                                               INS (novel insertions), INV (inversions), DUP:TANDEM (tandem duplications), \
                                               DUP:INT (interspersed duplications), BND (breakends).')
-    group_fasta_output.add_argument('--sequence_alleles',
+    group_fasta_output.add_argument('--symbolic_alleles',
                                         action='store_true',
-                                        help='Use nucleotide sequences for alleles of deletions, inversions and insertions in output VCF (default: %(default)s). \
-                                              By default, all SVs are represented by symbolic alleles, such as <DEL>, <INV> or <INS>. \
-                                              If enabled, ALT alleles of insertions are obtained from the sequence of a random read that supports the variant.')
+                                        help='Use symbolic alleles, such as <DEL> or <INV> in output VCF (default: %(default)s). \
+                                              By default, all SV alleles are represented by nucleotide sequences.')
     group_fasta_output.add_argument('--insertion_sequences',
                                         action='store_true',
                                         help='Output insertion sequences in INFO tag of VCF (default: %(default)s). \
-                                              If enabled, the INFO/SEQS tag contains a list of insertion sequences from the supporting reads. \
-                                              However, the insertion sequences are not combined into a consensus sequence.')
+                                              If enabled, the INFO/SEQS tag contains a list of insertion sequences from the supporting reads.')
     group_fasta_output.add_argument('--tandem_duplications_as_insertions',
                                         action='store_true',
                                         help='Represent tandem duplications as insertions in output VCF (default: %(default)s). \
@@ -307,7 +327,7 @@ Alternatively, it can detect SVs from existing reads alignments in SAM/BAM forma
                                             determines the maximum distance between two subsequent signatures in the same partition. \
                                             If the distance between two subsequent signatures \
                                             is larger than this parameter, they are distributed into separate partitions.')
-    group_bam_cluster.add_argument('--distance_normalizer',
+    group_bam_cluster.add_argument('--position_distance_normalizer',
                                       type=int,
                                       default=900,
                                       help='Distance normalizer used for span-position distance (default: %(default)s). \
@@ -322,9 +342,24 @@ Alternatively, it can detect SVs from existing reads alignments in SAM/BAM forma
                                             A smaller distance normalizer leads to a higher position distance and as a \
                                             consequence increases the importance of the position distance in the \
                                             span-position distance relative to the span distance.')
+    group_bam_cluster.add_argument('--edit_distance_normalizer',
+                                      type=float,
+                                      default=1.0,
+                                      help='Distance normalizer used specifically for insertions (default: %(default)s). \
+                                            SVIM clusters insertion signatures using an hierarchical clustering approach and a \
+                                            special distance metric for insertions. This distance is the sum of two \
+                                            components, position distance and edit distance between the insertion sequences. \
+                                            The edit distance is normalized (i.e. divided) by the product of the span of the \
+                                            longer insertion and this normalizer. The position distance is the difference in \
+                                            position between signatures normalized by the position distance normalizer \
+                                            (another parameter). A smaller edit distance normalizer leads to a larger \
+                                            edit distance and as a consequence increases the importance of the edit distance \
+                                            in the clustering process so that only insertions with very similar sequences \
+                                            are clustered together. A larger edit distance normalizer diminishes the importance \
+                                            of the insertion sequences in the clustering process.')
     group_bam_cluster.add_argument('--cluster_max_distance',
                                       type=float,
-                                      default=0.3,
+                                      default=0.5,
                                       help='Maximum span-position distance between SVs in a cluster (default: %(default)s). \
                                             This is the most important parameter because it determines the strictness \
                                             of clustering. Choosing a large value leads to fewer but larger clusters with larger \
@@ -346,18 +381,25 @@ Alternatively, it can detect SVs from existing reads alignments in SAM/BAM forma
                                       type=float,
                                       default=1.0,
                                       help='Maximum span-position distance between the origin of an insertion and a deletion to be flagged as a potential cut&paste insertion (default: %(default)s)')
-    group_bam_combine.add_argument('--trans_destination_partition_max_distance',
-                                      type=int,
-                                      default=1000,
-                                      help='Maximum distance in bp between translocation breakpoint destinations in a partition (default: %(default)s)')
-    group_bam_combine.add_argument('--trans_partition_max_distance',
-                                      type=int,
-                                      default=200,
-                                      help='Maximum distance in bp between translocation breakpoints in a partition (default: %(default)s)')
     group_bam_combine.add_argument('--trans_sv_max_distance',
                                       type=int,
                                       default=500,
                                       help='Maximum distance in bp between a translocation breakpoint and an SV signature to be combined (default: %(default)s)')
+    group_bam_combine.add_argument('--skip_consensus',
+                                    action='store_true',
+                                    help='Disable consensus computation for insertions (default: %(default)s). \
+                                          This reduces the time and memory consumption of SVIM and might be \
+                                          useful if consensus sequences are not needed. With this option, \
+                                          insertion calls are represented by symbolic alleles (<INS>) instead \
+                                          of sequence alles in the output VCF. \
+                                          Consensus computation requires a modern CPU with the SSE 4.1 \
+                                          instruction set. For older CPUs missing this instruction set, \
+                                          consensus computation is automatically disabled.')
+    group_bam_combine.add_argument('--max_consensus_length',
+                                        type=int,
+                                        default=10000,
+                                        help='Maximum size of insertion sequences for consensus computation. (default: %(default)s)\
+                                              For insertions longer than this threshold, no consensus is computed to save memory.')
     group_bam_genotype = parser_bam.add_argument_group('GENOTYPE')
     group_bam_genotype.add_argument('--skip_genotyping',
                                         action='store_true',
@@ -404,16 +446,14 @@ Alternatively, it can detect SVs from existing reads alignments in SAM/BAM forma
                                               Give a comma-separated list of SV types. The possible SV types are: DEL (deletions), \
                                               INS (novel insertions), INV (inversions), DUP:TANDEM (tandem duplications), \
                                               DUP:INT (interspersed duplications), BND (breakends).')
-    group_bam_output.add_argument('--sequence_alleles',
+    group_bam_output.add_argument('--symbolic_alleles',
                                         action='store_true',
-                                        help='Use nucleotide sequences for alleles of deletions, inversions and insertions in output VCF (default: %(default)s). \
-                                              By default, all SVs are represented by symbolic alleles, such as <DEL>, <INV> or <INS>. \
-                                              If enabled, ALT alleles of insertions are obtained from the sequence of a random read that supports the variant.')
+                                        help='Use symbolic alleles, such as <DEL> or <INV> in output VCF (default: %(default)s). \
+                                              By default, all SV alleles are represented by nucleotide sequences.')
     group_bam_output.add_argument('--insertion_sequences',
                                         action='store_true',
                                         help='Output insertion sequences in INFO tag of VCF (default: %(default)s). \
-                                              If enabled, the INFO/SEQS tag contains a list of insertion sequences from the supporting reads. \
-                                              However, the insertion sequences are not combined into a consensus sequence.')
+                                              If enabled, the INFO/SEQS tag contains a list of insertion sequences from the supporting reads.')
     group_bam_output.add_argument('--tandem_duplications_as_insertions',
                                         action='store_true',
                                         help='Represent tandem duplications as insertions in output VCF (default: %(default)s). \
