@@ -1,14 +1,21 @@
 from __future__ import print_function
 
 import sys
+import logging
 from statistics import mean
 
 from svim.SVSignature import SignatureDeletion, SignatureInsertion, SignatureInversion, SignatureTranslocation, SignatureDuplicationTandem, SignatureInsertionFrom
 from svim.SVIM_clustering import consolidate_clusters_bilocal, clusters_from_partitions
 
 
-def is_similar(chr1, start1, end1, chr2, start2, end2):
-    if chr1 == chr2 and abs(start1 - start2) < 20 and abs(end1 - end2) < 20:
+def is_similar(chr1, start1, end1, chr2, start2, end2, span_position_treshold = 0.3):
+    span1 = end1 - start1
+    span2 = end2 - start2
+    center1 = (start1 + end1) // 2
+    center2 = (start2 + end2) // 2
+    position_distance = abs(center1 - center2) / 900
+    span_distance = abs(span1 - span2) / max(span1, span2)
+    if chr1 == chr2 and position_distance + span_distance < span_position_treshold:
         return True
     else:
         return False
@@ -21,8 +28,12 @@ def analyze_read_segments(primary, supplementaries, bam, options):
     for alignment in alignments:
         #correct query coordinates for reversely mapped reads
         if alignment.is_reverse:
-            q_start = alignment.infer_read_length() - alignment.query_alignment_end
-            q_end = alignment.infer_read_length() - alignment.query_alignment_start
+            inferred_read_length = alignment.infer_read_length()
+            if inferred_read_length is None:
+                logging.warning('Skipping alignment because pysam was unable to infer length of read from CIGAR string (alignment.infer_read_length() returned None). Query name: {0}, CIGAR: {1}'.format(alignment.query_name, alignment.cigarstring))
+                continue
+            q_start = inferred_read_length - alignment.query_alignment_end
+            q_end = inferred_read_length - alignment.query_alignment_start
         else:
             q_start = alignment.query_alignment_start
             q_end = alignment.query_alignment_end
@@ -39,6 +50,8 @@ def analyze_read_segments(primary, supplementaries, bam, options):
     #inferred_read_length = alignments[0].infer_read_length()
 
     sv_signatures = []
+    #Translocation signatures from other SV classes are stored separately for --all_bnd option
+    translocation_signatures_all_bnds = []
     tandem_duplications = []
     translocations = []
 
@@ -86,11 +99,11 @@ def analyze_read_segments(primary, supplementaries, bam, options):
                                 if not alignment_current['is_reverse']:
                                     sv_signatures.append(SignatureDeletion(ref_chr, alignment_current['ref_end'], alignment_current['ref_end'] - deviation, "suppl", read_name))
                                     if options.all_bnds:
-                                        sv_signatures.append(SignatureTranslocation(ref_chr, alignment_current['ref_end'] - 1, 'fwd', ref_chr, alignment_current['ref_end'] - deviation, 'fwd', "suppl", read_name))
+                                        translocation_signatures_all_bnds.append(SignatureTranslocation(ref_chr, alignment_current['ref_end'] - 1, 'fwd', ref_chr, alignment_current['ref_end'] - deviation, 'fwd', "suppl", read_name))
                                 else:
                                     sv_signatures.append(SignatureDeletion(ref_chr, alignment_next['ref_end'], alignment_next['ref_end'] - deviation, "suppl", read_name))
                                     if options.all_bnds:
-                                        sv_signatures.append(SignatureTranslocation(ref_chr, alignment_next['ref_end'] - 1, 'fwd', ref_chr, alignment_next['ref_end'] - deviation, 'fwd', "suppl", read_name))
+                                        translocation_signatures_all_bnds.append(SignatureTranslocation(ref_chr, alignment_next['ref_end'] - 1, 'fwd', ref_chr, alignment_next['ref_end'] - deviation, 'fwd', "suppl", read_name))
                         #Either very large DEL or TRANS
                         elif deviation < -options.max_sv_size:
                             #No gap on read
@@ -110,12 +123,12 @@ def analyze_read_segments(primary, supplementaries, bam, options):
                                 if alignment_next['ref_end'] > alignment_current['ref_start']:
                                     tandem_duplications.append((ref_chr, alignment_next['ref_start'], alignment_current['ref_end'], True, True))
                                     if options.all_bnds:
-                                        sv_signatures.append(SignatureTranslocation(ref_chr, alignment_current['ref_end'] - 1, 'fwd', ref_chr, alignment_next['ref_start'], 'fwd', "suppl", read_name))
+                                        translocation_signatures_all_bnds.append(SignatureTranslocation(ref_chr, alignment_current['ref_end'] - 1, 'fwd', ref_chr, alignment_next['ref_start'], 'fwd', "suppl", read_name))
                                 #Large tandem duplication
                                 elif distance_on_reference >= -options.max_sv_size:
                                     tandem_duplications.append((ref_chr, alignment_next['ref_start'], alignment_current['ref_end'], False, True))
                                     if options.all_bnds:
-                                        sv_signatures.append(SignatureTranslocation(ref_chr, alignment_current['ref_end'] - 1, 'fwd', ref_chr, alignment_next['ref_start'], 'fwd', "suppl", read_name))
+                                        translocation_signatures_all_bnds.append(SignatureTranslocation(ref_chr, alignment_current['ref_end'] - 1, 'fwd', ref_chr, alignment_next['ref_start'], 'fwd', "suppl", read_name))
                                 #Either very large TANDEM or TRANS
                                 else:
                                     sv_signatures.append(SignatureTranslocation(ref_chr, alignment_current['ref_end'] - 1, 'fwd', ref_chr, alignment_next['ref_start'], 'fwd', "suppl", read_name))
@@ -125,12 +138,12 @@ def analyze_read_segments(primary, supplementaries, bam, options):
                                 if alignment_next['ref_start'] < alignment_current['ref_end']:
                                     tandem_duplications.append((ref_chr, alignment_current['ref_start'], alignment_next['ref_end'], True, False))
                                     if options.all_bnds:
-                                        sv_signatures.append(SignatureTranslocation(ref_chr, alignment_current['ref_start'], 'rev', ref_chr, alignment_next['ref_end'] - 1, 'rev', "suppl", read_name))
+                                        translocation_signatures_all_bnds.append(SignatureTranslocation(ref_chr, alignment_current['ref_start'], 'rev', ref_chr, alignment_next['ref_end'] - 1, 'rev', "suppl", read_name))
                                 #Large tandem duplication
                                 elif distance_on_reference >= -options.max_sv_size:
                                     tandem_duplications.append((ref_chr, alignment_current['ref_start'], alignment_next['ref_end'], False, False))
                                     if options.all_bnds:
-                                        sv_signatures.append(SignatureTranslocation(ref_chr, alignment_current['ref_start'], 'rev', ref_chr, alignment_next['ref_end'] - 1, 'rev', "suppl", read_name))
+                                        translocation_signatures_all_bnds.append(SignatureTranslocation(ref_chr, alignment_current['ref_start'], 'rev', ref_chr, alignment_next['ref_end'] - 1, 'rev', "suppl", read_name))
                                 #Either very large TANDEM or TRANS
                                 else:
                                     sv_signatures.append(SignatureTranslocation(ref_chr, alignment_current['ref_start'], 'rev', ref_chr, alignment_next['ref_end'] - 1, 'rev', "suppl", read_name))
@@ -145,7 +158,7 @@ def analyze_read_segments(primary, supplementaries, bam, options):
                             if options.min_sv_size <= alignment_next['ref_end'] - alignment_current['ref_end'] <= options.max_sv_size:
                                 sv_signatures.append(SignatureInversion(ref_chr, alignment_current['ref_end'], alignment_next['ref_end'], "suppl", read_name, "left_fwd"))
                                 if options.all_bnds:
-                                    sv_signatures.append(SignatureTranslocation(ref_chr, alignment_current['ref_end'] - 1, 'fwd', ref_chr, alignment_next['ref_end'] - 1, 'rev', "suppl", read_name))
+                                    translocation_signatures_all_bnds.append(SignatureTranslocation(ref_chr, alignment_current['ref_end'] - 1, 'fwd', ref_chr, alignment_next['ref_end'] - 1, 'rev', "suppl", read_name))
                             #Either very large INV or TRANS
                             elif alignment_next['ref_end'] - alignment_current['ref_end'] > options.max_sv_size:
                                 sv_signatures.append(SignatureTranslocation(ref_chr, alignment_current['ref_end'] - 1, 'fwd', ref_chr, alignment_next['ref_end'] - 1, 'rev', "suppl", read_name))
@@ -155,7 +168,7 @@ def analyze_read_segments(primary, supplementaries, bam, options):
                             if options.min_sv_size <= alignment_current['ref_end'] - alignment_next['ref_end'] <= options.max_sv_size:
                                 sv_signatures.append(SignatureInversion(ref_chr, alignment_next['ref_end'], alignment_current['ref_end'], "suppl", read_name, "left_rev"))
                                 if options.all_bnds:
-                                    sv_signatures.append(SignatureTranslocation(ref_chr, alignment_current['ref_end'] - 1, 'fwd', ref_chr, alignment_next['ref_end'] - 1, 'rev', "suppl", read_name))
+                                    translocation_signatures_all_bnds.append(SignatureTranslocation(ref_chr, alignment_current['ref_end'] - 1, 'fwd', ref_chr, alignment_next['ref_end'] - 1, 'rev', "suppl", read_name))
                             #Either very large INV or TRANS
                             elif alignment_current['ref_end'] - alignment_next['ref_end'] > options.max_sv_size:
                                 sv_signatures.append(SignatureTranslocation(ref_chr, alignment_current['ref_end'] - 1, 'fwd', ref_chr, alignment_next['ref_end'] - 1, 'rev', "suppl", read_name))
@@ -171,7 +184,7 @@ def analyze_read_segments(primary, supplementaries, bam, options):
                             if options.min_sv_size <= alignment_next['ref_start'] - alignment_current['ref_start'] <= options.max_sv_size:
                                 sv_signatures.append(SignatureInversion(ref_chr, alignment_current['ref_start'], alignment_next['ref_start'], "suppl", read_name, "right_fwd"))
                                 if options.all_bnds:
-                                    sv_signatures.append(SignatureTranslocation(ref_chr, alignment_current['ref_start'], 'rev', ref_chr, alignment_next['ref_start'], 'fwd', "suppl", read_name))
+                                    translocation_signatures_all_bnds.append(SignatureTranslocation(ref_chr, alignment_current['ref_start'], 'rev', ref_chr, alignment_next['ref_start'], 'fwd', "suppl", read_name))
                             #Either very large INV or TRANS
                             elif alignment_next['ref_start'] - alignment_current['ref_start'] > options.max_sv_size:
                                 sv_signatures.append(SignatureTranslocation(ref_chr, alignment_current['ref_start'], 'rev', ref_chr, alignment_next['ref_start'], 'fwd', "suppl", read_name))
@@ -181,7 +194,7 @@ def analyze_read_segments(primary, supplementaries, bam, options):
                             if options.min_sv_size <= alignment_current['ref_start'] - alignment_next['ref_start'] <= options.max_sv_size:
                                 sv_signatures.append(SignatureInversion(ref_chr, alignment_next['ref_start'], alignment_current['ref_start'], "suppl", read_name, "right_rev"))
                                 if options.all_bnds:
-                                    sv_signatures.append(SignatureTranslocation(ref_chr, alignment_current['ref_start'], 'rev', ref_chr, alignment_next['ref_start'], 'fwd', "suppl", read_name))
+                                    translocation_signatures_all_bnds.append(SignatureTranslocation(ref_chr, alignment_current['ref_start'], 'rev', ref_chr, alignment_next['ref_start'], 'fwd', "suppl", read_name))
                             #Either very large INV or TRANS
                             elif alignment_current['ref_start'] - alignment_next['ref_start'] > options.max_sv_size:
                                 sv_signatures.append(SignatureTranslocation(ref_chr, alignment_current['ref_start'], 'rev', ref_chr, alignment_next['ref_start'], 'fwd', "suppl", read_name))
@@ -271,7 +284,7 @@ def analyze_read_segments(primary, supplementaries, bam, options):
             #Same direction at destination and origin
             if before_dir1 == this_dir2 and before_dir2 == this_dir1:
                 #Same position at destination
-                if is_similar(before_chr1, before_pos1, 0, this_chr2, this_pos2, 0):
+                if is_similar(before_chr1, before_pos1, before_pos1+1, this_chr2, this_pos2, this_pos2+1, span_position_treshold=0.1):
                     #Same chromosome for origin
                     if before_chr2 == this_chr1:
                         #INS_DUP candidate
@@ -286,4 +299,4 @@ def analyze_read_segments(primary, supplementaries, bam, options):
                         else:
                             pass
 
-    return sv_signatures
+    return sv_signatures, translocation_signatures_all_bnds
